@@ -41,8 +41,6 @@
 -include("s1ap.hrl").
 
 -define(SERVER, ?MODULE).
--define(ENV_DEFAULT_SCTP_BUFSZ, 65536).
--define(ENV_DEFAULT_SCTP_NODELAY, true).
 -define(ENV_DEFAULT_S1GW_BIND_ADDR, "127.0.1.1").
 -define(ENV_DEFAULT_S1GW_BIND_PORT, ?S1AP_PORT).
 -define(ENV_DEFAULT_MME_LOC_ADDR, "127.0.2.1").
@@ -103,26 +101,44 @@ init([]) ->
 %% private API
 %% ------------------------------------------------------------------
 
+-spec parse_env_map(map()) -> map().
+parse_env_map(M0) ->
+    M1 = maps:map(fun(_, P) -> osmo_s1gw:get_env(P, undefined) end, M0),
+    maps:filter(fun(_, V) -> V =/= undefined end, M1).
+
 
 -spec client_cfg() -> sctp_client:cfg().
 client_cfg() ->
-    SockOpts = #{sctp_nodelay => osmo_s1gw:get_env(mme_nodelay, ?ENV_DEFAULT_SCTP_NODELAY),
-                 recbuf => osmo_s1gw:get_env(mme_recbuf, ?ENV_DEFAULT_SCTP_BUFSZ),
-                 sndbuf => osmo_s1gw:get_env(mme_sndbuf, ?ENV_DEFAULT_SCTP_BUFSZ)},
-    #{laddr => osmo_s1gw:get_env(mme_loc_addr, ?ENV_DEFAULT_MME_LOC_ADDR),
-      raddr => osmo_s1gw:get_env(mme_rem_addr, ?ENV_DEFAULT_MME_REM_ADDR),
-      rport => osmo_s1gw:get_env(mme_rem_port, ?ENV_DEFAULT_MME_REM_PORT),
-      sockopts => maps:to_list(SockOpts)}.
+    %% parse old/legacy environment options, if any
+    OldCfg = parse_env_map(#{laddr => mme_loc_addr,
+                             raddr => mme_rem_addr,
+                             rport => mme_rem_port}),
+    OldSockOpts = parse_env_map(#{nodelay => mme_nodelay,
+                                  recbuf => mme_recbuf,
+                                  sndbuf => mme_sndbuf}),
+    %% parse the new sctp_client configuration block
+    Cfg = maps:merge(OldCfg, osmo_s1gw:get_env(sctp_client, #{ })),
+    SockOpts = maps:merge(OldSockOpts, maps:get(sockopts, Cfg, #{ })),
+    #{laddr => maps:get(laddr, Cfg, ?ENV_DEFAULT_MME_LOC_ADDR),
+      raddr => maps:get(raddr, Cfg, ?ENV_DEFAULT_MME_REM_ADDR),
+      rport => maps:get(rport, Cfg, ?ENV_DEFAULT_MME_REM_PORT),
+      sockopts => sctp_common:gen_sockopts(SockOpts)}.
 
 
 -spec server_cfg() -> sctp_server:cfg().
 server_cfg() ->
-    SockOpts = #{sctp_nodelay => osmo_s1gw:get_env(s1gw_nodelay, ?ENV_DEFAULT_SCTP_NODELAY),
-                 recbuf => osmo_s1gw:get_env(s1gw_recbuf, ?ENV_DEFAULT_SCTP_BUFSZ),
-                 sndbuf => osmo_s1gw:get_env(s1gw_sndbuf, ?ENV_DEFAULT_SCTP_BUFSZ)},
-    #{laddr => osmo_s1gw:get_env(s1gw_bind_addr, ?ENV_DEFAULT_S1GW_BIND_ADDR),
-      lport => osmo_s1gw:get_env(s1gw_bind_port, ?ENV_DEFAULT_S1GW_BIND_PORT),
-      sockopts => maps:to_list(SockOpts),
+    %% parse old/legacy environment options, if any
+    DefCfg = parse_env_map(#{laddr => s1gw_bind_addr,
+                             lport => s1gw_bind_port}),
+    DefSockOpts = parse_env_map(#{nodelay => s1gw_nodelay,
+                                  recbuf => s1gw_recbuf,
+                                  sndbuf => s1gw_sndbuf}),
+    %% parse the new sctp_server configuration block
+    Cfg = maps:merge(DefCfg, osmo_s1gw:get_env(sctp_server, #{ })),
+    SockOpts = maps:merge(DefSockOpts, maps:get(sockopts, Cfg, #{ })),
+    #{laddr => maps:get(laddr, Cfg, ?ENV_DEFAULT_S1GW_BIND_ADDR),
+      lport => maps:get(lport, Cfg, ?ENV_DEFAULT_S1GW_BIND_PORT),
+      sockopts => sctp_common:gen_sockopts(SockOpts),
       handler => sctp_proxy,
       priv => client_cfg()}.
 
