@@ -53,16 +53,28 @@
 
 -include("s1gw_metrics.hrl").
 
+
+-type conn_info() :: #{state := atom(),
+                       handler := pid(),
+                       mme_addr := inet:ip_address(),
+                       mme_port := inet:port_number(),
+                       enb_aid := gen_sctp:assoc_id(),
+                       mme_aid => gen_sctp:assoc_id()
+                      }.
+
+-export_type([conn_info/0]).
+
+
 %% ------------------------------------------------------------------
 %% public API
 %% ------------------------------------------------------------------
 
--spec start_link(Aid, ConnCfg) -> Result
-    when Aid :: gen_sctp:assoc_id(),
-         ConnCfg :: sctp_client:cfg(),
+-spec start_link(ConnInfo, Priv) -> Result
+    when ConnInfo :: sctp_server:conn_info(),
+         Priv :: sctp_client:cfg(),
          Result :: gen_statem:start_ret().
-start_link(Aid, ConnCfg) ->
-    gen_statem:start_link(?MODULE, [Aid, ConnCfg], []).
+start_link(ConnInfo, Priv) ->
+    gen_statem:start_link(?MODULE, [ConnInfo, Priv], []).
 
 
 -spec send_data(pid(), binary()) -> ok.
@@ -70,7 +82,7 @@ send_data(Pid, Data) ->
     gen_statem:cast(Pid, {send_data, Data}).
 
 
--spec fetch_info(pid()) -> proplists:proplist().
+-spec fetch_info(pid()) -> conn_info().
 fetch_info(Pid) ->
     gen_statem:call(Pid, ?FUNCTION_NAME).
 
@@ -84,12 +96,15 @@ shutdown(Pid) ->
 %% gen_statem API
 %% ------------------------------------------------------------------
 
-init([Aid, ConnCfg]) ->
-    {ok, Pid} = s1ap_proxy:start_link(),
+init([EnbConnInfo, MmeConnCfg]) ->
+    {ok, EnbHandle} = enb_registry:enb_register(),
+    {ok, Pid} = s1ap_proxy:start_link(EnbHandle, EnbConnInfo),
     {ok, connecting,
-     #{enb_aid => Aid,
-       conn_cfg => ConnCfg,
+     #{enb_aid => maps:get(aid, EnbConnInfo),
+       enb_conn_info => EnbConnInfo,
+       mme_conn_cfg => MmeConnCfg,
        tx_queue => [],
+       enb_handle => EnbHandle,
        handler => Pid}}.
 
 
@@ -99,10 +114,13 @@ callback_mode() ->
 
 %% CONNECTING state
 connecting(enter, OldState,
-           #{conn_cfg := ConnCfg} = S) ->
+           #{enb_conn_info := EnbConnInfo,
+             mme_conn_cfg := MmeConnCfg,
+             enb_handle := Handle} = S) ->
     ?LOG_INFO("State change: ~p -> ~p", [OldState, ?FUNCTION_NAME]),
+    ok = enb_registry:enb_event(Handle, {?FUNCTION_NAME, EnbConnInfo}),
     %% Initiate connection establishment with the MME
-    {ok, Sock} = sctp_client:connect(ConnCfg),
+    {ok, Sock} = sctp_client:connect(MmeConnCfg),
     {next_state, connecting, S#{sock => Sock},
      [{state_timeout, 2_000, conn_est_timeout}]};
 
@@ -131,20 +149,16 @@ connecting(info, {sctp, _Socket, MmeAddr, MmePort,
             {stop, {shutdown, conn_est_fail}}
     end;
 
-%% Catch-all for other kinds of SCTP events
-connecting(info, {sctp, _Socket, MmeAddr, MmePort,
-                  {AncData, Data}}, S) ->
-    ?LOG_DEBUG("Unhandled SCTP event (~p:~p): ~p, ~p",
-               [MmeAddr, MmePort, AncData, Data]),
-    {keep_state, S};
-
 connecting(Event, EventData, S) ->
     handle_event(?FUNCTION_NAME, Event, EventData, S).
 
 
 %% CONNECTED state
-connected(enter, OldState, S0) ->
+connected(enter, OldState,
+          #{enb_handle := Handle} = S0) ->
     ?LOG_INFO("State change: ~p -> ~p", [OldState, ?FUNCTION_NAME]),
+    MmeConnInfo = conn_info(?FUNCTION_NAME, S0),
+    ok = enb_registry:enb_event(Handle, {?FUNCTION_NAME, MmeConnInfo}),
     %% Send pending eNB -> MME messages (if any)
     S1 = sctp_send_pending(S0),
     {keep_state, S1};
@@ -186,34 +200,37 @@ connected(info, {sctp, _Socket, MmeAddr, MmePort,
         {forward, FwdData} ->
             sctp_server:send_data(EnbAid, FwdData);
         {reply, ReData} ->
-            ok = sctp_common:send_data({Sock, Aid}, ReData)
+            ok = sctp_common:send_data({Sock, Aid}, ReData);
+        {drop, Data} ->
+            ok %% no-op
     end,
     {keep_state, S};
 
-%% Catch-all for other kinds of SCTP events
-connected(info, {sctp, _Socket, MmeAddr, MmePort,
-                 {AncData, Data}}, S) ->
-    ?LOG_DEBUG("Unhandled SCTP event (~p:~p): ~p, ~p",
-               [MmeAddr, MmePort, AncData, Data]),
-    {keep_state, S};
-
 connected(Event, EventData, S) ->
     handle_event(?FUNCTION_NAME, Event, EventData, S).
 
 
 %% Event handler for all states
-handle_event(State, {call, From}, fetch_info,
-             #{handler := Pid} = S0) ->
-    ERABs = s1ap_proxy:fetch_erab_list(Pid),
-    S1 = maps:with([enb_aid, mme_aid, mme_addr, mme_port], S0),
-    S2 = S1#{state => State, erab_list => ERABs},
-    Reply = proplists:from_map(S2),
-    {keep_state_and_data, {reply, From, Reply}};
+handle_event(State, {call, From}, fetch_info, S) ->
+    Info = conn_info(State, S),
+    {keep_state_and_data, {reply, From, Info}};
 
 handle_event(State, {call, From}, EventData, _S) ->
     ?LOG_ERROR("Unexpected call in state ~p: ~p", [State, EventData]),
     {keep_state_and_data, {reply, From, {error, {unexpected_call, State}}}};
 
+%% Catch-all for unhandled SCTP events
+handle_event(State, info, {sctp, _Socket, MmeAddr, MmePort, {AncData, Data}}, _S) ->
+    ?LOG_DEBUG("Unhandled SCTP event in state ~p (~p:~p): ~p, ~p",
+               [State, MmeAddr, MmePort, AncData, Data]),
+    keep_state_and_data;
+
+handle_event(State, info, {sctp_error, _Socket, MmeAddr, MmePort, {_, Error}}, _S) ->
+    ?LOG_ERROR("Rx SCTP error in state ~p (~p:~p): ~p",
+               [State, MmeAddr, MmePort, Error]),
+    sctp_common:report_error(Error),
+    keep_state_and_data;
+
 handle_event(State, Event, EventData, _S) ->
     ?LOG_ERROR("Unexpected event ~p in state ~p: ~p", [Event, State, EventData]),
     keep_state_and_data.
@@ -223,8 +240,11 @@ code_change(_Vsn, State, S, _Extra) ->
     {ok, State, S}.
 
 
-terminate(Reason, State, #{handler := Pid} = S) ->
+terminate(Reason, State,
+          #{handler := Pid,
+            enb_handle := Handle} = S) ->
     ?LOG_NOTICE("Terminating in state ~p, reason ~p", [State, Reason]),
+    enb_registry:enb_unregister(Handle),
     s1ap_proxy:shutdown(Pid),
     case S of
         #{sock := Sock,
@@ -251,7 +271,9 @@ sctp_send(Data,
         {forward, FwdData} ->
             ok = sctp_common:send_data({Sock, Aid}, FwdData);
         {reply, ReData} ->
-            sctp_server:send_data(EnbAid, ReData)
+            sctp_server:send_data(EnbAid, ReData);
+        {drop, Data} ->
+            ok %% no-op
     end.
 
 
@@ -267,4 +289,12 @@ sctp_send_pending([Data | Pending], S) ->
 sctp_send_pending([], S) ->
     S#{tx_queue := []}.
 
+
+conn_info(State, #{mme_conn_cfg := MmeConnCfg} = S0) ->
+    S1 = maps:with([enb_aid, mme_aid, handler], S0),
+    S1#{state => State,
+        mme_addr => maps:get(raddr, MmeConnCfg),
+        mme_port => maps:get(rport, MmeConnCfg)}.
+
+
 %% vim:set ts=4 sw=4 et:
